import pycurl
import urllib
import xmltodict
try:
    from io import BytesIO
except ImportError:
    from StringIO import StringIO as BytesIO

## @package ardi.api.relations
#  ARDI Relationship API
#
#  This class allows you to create a service (that runs as a daemon on Linux or via NSSM on Windows) that
#  responds to live data.
#
#  @todo This class is currently incomplete and only offers limited functionality.

## ARDI Relationships
#
#  Used to interact with ARDI relationships, allowing you to connect and disconnect assets from eachother.
class ARDIRelations:
    def __init__(self, core):
        self.core = core

    ## Creates a relationship
    #
    #  Connects two assets together via a relationship.
    #
    #  @param source The parent in the relationship.
    #  @param destination The child in the relationship
    #  @param relationship The ID number of the relationship, available from the ARDICore class.
    #
    #  @returns TRUE if the change has been made, or FALSE if there was an error either in communication or from ARDI.
    def Link(self,source,destination,relationship):

        final = urllib.urlencode({ 'origin' : source, 'destination' : destination, 'rel' : relationship })

        resp = []

        buffer = BytesIO()
        ar = pycurl.Curl()
        ar.setopt(ar.URL, 'http://' + self.core.server + ':' + str(self.core.port) + '/api/linkassets.php?' + final)
        ar.setopt(ar.WRITEFUNCTION, buffer.write)
        ar.perform()
        
        # HTTP response code, e.g. 200.
        if ar.getinfo(ar.RESPONSE_CODE) == 200:    
            xml = xmltodict.parse(buffer.getvalue())

            ar.close()
            
            try:
                res = xml['response']['success']
                return True
            except:
                return False            
            
        ar.close()
        return False
    
    ## Deletes a relationship
    #
    #  Disconnects two assets by erasing the relationship between them.
    #
    #  @param source The first asset in the relationship.
    #  @param destination The second asset in the relationship.
    #  @param relationship The ID number of the relationship, available from the ARDICore class.
    #
    #  @returns TRUE if the change has been made, or FALSE if there was an error either in communication or from ARDI.
    def Unlink(self,source,destination,relationship):

        final = urllib.urlencode({ 'origin' : source, 'destination' : destination, 'rel' : relationship })

        resp = []

        buffer = BytesIO()
        ar = pycurl.Curl()
        ar.setopt(ar.URL, 'http://' + self.core.server + ':' + str(self.core.port) + '/api/unlinkassets.php?' + final)
        ar.setopt(ar.WRITEFUNCTION, buffer.write)
        ar.perform()
        
        # HTTP response code, e.g. 200.
        if ar.getinfo(ar.RESPONSE_CODE) == 200:    
            xml = xmltodict.parse(buffer.getvalue())

            ar.close()

            print xml;
            try:
                res = xml['response']['success']
                return True
            except:
                return False            
            
        ar.close()
        return False
